#include "skein/particlemap.h"
#include "skein/particle.h"


const Particle& ParticleMap::getParticle(const std::string& id) const
{
    return _particles.at(id);
}

const Orbit& ParticleMap::getOrbit(const std::string& id) const
{
    return _orbits.at(id);
}

glm::dvec3 ParticleMap::getParticlePosition(const std::string& id, double time) const
{
    if (_orbits.find(id) == _orbits.end())
        return {0,0,0};

    // TODO: actually get stuff based on physics
    //const Particle& parent = _relationships.at(id);

    // how do we get the gravitational parameter?
    // is it just G * M
    // where M is the mass of the parent
    //const double u = getGravitationalParameter(parent.getId());
    //const double mass = parent.getMass();

    // how do we get the particle position from the time + gravitational parameter?

    // TODO: actually nest stuff so position is determined from all parents
    const Orbit& orbit = _orbits.at(id);
    return orbit.getPosition(time);
}

void ParticleMap::setParticle(const Particle& particle)
{
    _particles.insert({particle.getId(), particle});
}

void ParticleMap::setRelationship(const std::string& parentId, const std::string& childId, const Orbit& orbit)
{
    // map children to parent - children can only have one parent, so we use the map to
    // identify the parent directly using the child as the key
    _relationships.insert({childId, parentId});
    _orbits.insert({childId, orbit});
}
